"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.process = process;

var _reactNative = require("react-native");

var _cssMediaquery = require("css-mediaquery");

var _cssMediaquery2 = _interopRequireDefault(_cssMediaquery);

var _types = require("./types");

var _features = require("./features");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PREFIX = "@media";
var lengthRe = /^(0$|(?:[+-]?(?:\d*\.)?\d+(?:[Ee][+-]?\d+)?)(?=px|rem$))/;

function isObject(obj) {
  return (typeof obj === "undefined" ? "undefined" : _typeof(obj)) === "object" && obj !== null;
}

function isMediaQuery(str) {
  return typeof str === "string" && str.toLowerCase().indexOf(PREFIX) === 0;
}

function process(obj) {
  var mqKeys = [];

  var res = Object.keys(obj).reduce(function (res, k) {
    var key = k.trim();
    if (!isMediaQuery(key)) {
      res[key] = obj[key];
    } else {
      mqKeys.push(k);
    }
    return res;
  }, {});

  if (mqKeys.length) {
    var matchObject = getMatchObject();
    mqKeys.forEach(function (k) {
      var key = k.trim();
      var mqStr = key.toLowerCase().replace(PREFIX, "");
      var parsed = _cssMediaquery2.default.parse(mqStr);

      parsed.forEach(function (mq) {
        if (_types.mediaQueryTypes.indexOf(mq.type) === -1) {
          throw new Error("Failed to parse media query type \"" + mq.type + "\"");
        }

        mq.expressions.forEach(function (e) {
          var mf = e.modifier ? e.modifier + "-" + e.feature : e.feature;
          var val = e.value ? ": " + e.value : "";

          if (_features.mediaQueryFeatures.indexOf(e.feature) === -1) {
            throw new Error("Failed to parse media query feature \"" + mf + "\"");
          }

          if (_features.dimensionFeatures.indexOf(e.feature) > -1 && lengthRe.test(e.value) === false) {
            throw new Error("Failed to parse media query expression \"(" + mf + val + ")\"");
          }
        });
      });

      if (/^@media\s+(ios|android)/i.test(key)) {
        matchObject.type = _reactNative.Platform.OS;
      }

      var isMatch = _cssMediaquery2.default.match(mqStr, matchObject);
      if (isMatch) {
        merge(res, obj[k]);
      }
    });
  }

  return res;
}

function getMatchObject() {
  var win = _reactNative.Dimensions.get("window");
  return {
    width: win.width,
    height: win.height,
    orientation: win.width > win.height ? "landscape" : "portrait",
    "aspect-ratio": win.width / win.height,
    type: "screen"
  };
}

function merge(obj, mqObj) {
  Object.keys(mqObj).forEach(function (k) {
    var key = k.trim();
    if (isObject(obj[key]) && isObject(mqObj[key])) {
      Object.assign(obj[key], mqObj[key]);
    } else {
      obj[key] = mqObj[key];
    }
  });
}